(* by Bruce F. Torrence, Last Modified 9/19/2004 *)

BeginPackage["RMCPackages`CircularPlot3D`","Utilities`FilterOptions`"]

Center::usage = "Center is an option for CircularPlot3D. Set Center to 
the point about which the circular domain should be centered. For 
example, a typical setting might be Center\[Rule]{1,2}." ;

CircularPlot3D::usage = "CircularPlot3D[f,{r,rmin,rmax}] will produce 
a 3D plot of the real valued function f. The function f may be given either in the 
rectangular variables x and y or in the polar variables r and 
\[Theta]). The domain will be a 
circle centered at the origin (or centered 
at the point specified by the option Center). A typical setting might
for the Center option might be 
Center\[Rule]{1,2}. CircularPlot3D will accept a third argument: 
CircularPlot3D[f,{r,rmin,rmax},{\[Theta],\[Theta]min,\[Theta]max}] will 
only produce that portion of the plot with polar angle \[Theta] in the 
specified range. CircluarPlot3D will accept any options 
accepted by ParametricPlot3D.
Note: When f is given in rectangular coordinates, it is assumed that
(1) you are using the variable names x and y in your expression f, and (2)
that you are working in the Global` context. If this is not the case, you
should use the more formal syntax 
CircularPlot3D[f,{xvar,yvar},{r, rmin, rmax}], where xvar and yvar indicate
the names of the variables appearing in f, to declare the variable names
you are using. Similarly, if f is given in cylindrical coordinates it
is assumed that the angular variable name is \[Theta] and that you are
working in the global context when you enter the command
CircularPlot3D[f,{r,rmin,rmax}]. If this is not the case, you should 
use the more formal syntax CircularPlot3D[f,{a},{r,rmin,rmax}], where
a is the name of the angular variable. This is unnecessary if an iterator
for the angular variable is specified.";

CircularPlot3D::badarg=
    "Oops! You have assigned a value to the symbol `1`. You must first clear this value by typing Clear[`1`] before using CircularPlot3D.";
    


Options[CircularPlot3D]={
	Center->{0,0},
	PlotPoints->21,
	BoxRatios->{1,1,1}
	};


Begin["`Private`"]

(* RMCPackages`CircularPlot3D`Private` *)
 
CircularPlot3D[expr_,{x_,y_},{r_, rmin_, rmax_}, 
   {th_, thmin_, thmax_}, opts___] := 
  Module[{xc, yc}, 
  {xc, yc} = Center /. {opts} /. Options[CircularPlot3D]; 
  ParametricPlot3D[Evaluate[{x + xc, y + yc, 
        expr /. {x -> x + xc, y -> y + yc}} /. 
       {x -> r*Cos[th], y -> r*Sin[th]}], {r, rmin, rmax}, 
     {th, thmin, thmax}, 
     Evaluate[FilterOptions[ParametricPlot3D, opts]], 
     Evaluate[FilterOptions[ParametricPlot3D, Options[CircularPlot3D]]]]] 

CircularPlot3D[expr_,{th_},{r_, rmin_, rmax_}, 
   opts___] := 
  Module[{xc, yc}, 
  {xc, yc} = Center /. {opts} /. Options[CircularPlot3D]; 
  ParametricPlot3D[{r*Cos[th], r*Sin[th],expr}, {r, rmin, rmax}, 
     {th, 0, 2*Pi}, 
     Evaluate[FilterOptions[ParametricPlot3D, opts]], 
     Evaluate[FilterOptions[ParametricPlot3D, Options[CircularPlot3D]]]]] 

CircularPlot3D[expr_,{r_, rmin_, rmax_}, 
   {th_, thmin_, thmax_}, opts___] := 
 Module[{xc, yc}, 
  {xc, yc} = Center /. {opts} /. Options[CircularPlot3D]; 
  ParametricPlot3D[Evaluate[{Global`x + xc, Global`y + yc, 
        expr /. {Global`x -> Global`x + xc, Global`y -> Global`y + yc}} /. 
       {Global`x -> r*Cos[th], Global`y -> r*Sin[th]}], {r, rmin, rmax}, 
     {th, thmin, thmax}, 
     Evaluate[FilterOptions[ParametricPlot3D, opts]], 
     Evaluate[FilterOptions[ParametricPlot3D, Options[CircularPlot3D]]]]] 
 

CircularPlot3D[expr_,{x_,y_},{r_, rmin_, rmax_}, opts___] := 
 CircularPlot3D[expr,{x,y},{r,rmin,rmax},{th,0,2*Pi},opts] 

CircularPlot3D[expr_,{r_,rmin_,rmax_},opts___]:=
  If[ValueQ[Global`\[Theta]],Message[CircularPlot3D::badarg,"\[Theta]"],
  CircularPlot3D[expr,{r,rmin,rmax},{Global`\[Theta],0,2*Pi},opts]]

End[]

EndPackage[]